/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 27/02/2008
   derniere modification : 19/03/2008 */

#include <SCE/SCEMinimal.h>

#include <SCE/interface/SCEBoundingBox.h>

/* ajoute le 27/02/2008 */
/* revise le 01/03/2008 */
void SCE_BoundingBox_Init (SCE_SBoundingBox *box)
{
    int i;
    for (i=0; i<8; i++)
        SCE_Vector3_Operator1 (box->p[i], =, 0.);
    box->pushed = SCE_FALSE;
}

/* ajoute le 28/02/2008 */
/* revise le 01/03/2008 */
void SCE_BoundingBox_Set (SCE_SBoundingBox *box, SCE_TVector3 o,
                          float w, float h, float d)
{
    SCE_Vector3_Set (box->p[0], o[0],   o[1],   o[2]);
    SCE_Vector3_Set (box->p[1], o[0]+w, o[1],   o[2]);
    SCE_Vector3_Set (box->p[2], o[0]+w, o[1]+h, o[2]);
    SCE_Vector3_Set (box->p[3], o[0],   o[1]+h, o[2]);
    SCE_Vector3_Set (box->p[4], o[0],   o[1]+h, o[2]+d);
    SCE_Vector3_Set (box->p[5], o[0]+w, o[1]+h, o[2]+d);
    SCE_Vector3_Set (box->p[6], o[0]+w, o[1],   o[2]+d);
    SCE_Vector3_Set (box->p[7], o[0],   o[1],   o[2]+d);
}
void SCE_BoundingBox_Setv (SCE_SBoundingBox *bx, SCE_TVector3 o, SCE_TVector3 d)
{
    SCE_BoundingBox_Set (bx, o, d[0], d[1], d[2]);
}
/*
 *  4_____________5
 *  |\           |\
 *  | \          | \
 *  |  \         |  \
 *  |   \        |   \
 *  |    \3______|____\2
 *  |    |       |    |         z   y
 *  |7___|_______|6   |          \ |
 *   \   |        \   |           \|__ x
 *    \  |         \  |
 *     \ |          \ |
 *      \|___________\|
 *       0            1
 */


/* ajoute le 19/03/2008 */
void SCE_BoundingBox_GetCenterv (SCE_SBoundingBox *box, SCE_TVector3 center)
{
    center[0] = (box->p[1][0] - box->p[0][0]) / 2.0 + box->p[0][0];
    center[1] = (box->p[2][1] - box->p[1][1]) / 2.0 + box->p[1][1];
    center[2] = (box->p[7][2] - box->p[0][2]) / 2.0 + box->p[0][2];
}

/* ajoute le 28/02/2008 */
/* revise le 01/03/2008 */
float* SCE_BoundingBox_GetPoints (SCE_SBoundingBox *box)
{
    return box->p;
}
void SCE_BoundingBox_GetPointsv (SCE_SBoundingBox *box, SCE_TVector3 *p)
{
    int i;
    for (i=0; i<8; i++)
        SCE_Vector3_Copy (p[i], box->p[i]);
}

/* ajoute le 01/03/2008 */
void SCE_BoundingBox_Push (SCE_SBoundingBox *box, SCE_TMatrix4 m)
{
    if (!box->pushed)
    {
        int i;
        for (i=0; i<8; i++)
            SCE_Vector3_Copy (box->olds[i], box->p[i]);
        for (i=0; i<8; i++)
            SCE_Matrix4_MulV3 (m, box->p[i]);
        box->pushed = SCE_TRUE;
    }
}
/* ajoute le 01/03/2008 */
void SCE_BoundingBox_Pop (SCE_SBoundingBox *box)
{
    if (box->pushed)
    {
        int i;
        for (i=0; i<8; i++)
            SCE_Vector3_Copy (box->p[i], box->olds[i]);
        box->pushed = SCE_FALSE;
    }
}
